/*
 * comm.c
 *
 * Created: 2015-05-21 07:59:35
 *  Author: dsosnowski
 */ 

#define F_CPU 8000000UL
#include <avr/io.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "comm.h"
#include "ntctemp.h"
#include "ee_mem.h"
#include "s_uart.h"


struct {
	int16_t source;
	int16_t temp;	
	int16_t range;
	int16_t hyst;
	int16_t func;
	char buff[20];
} sett;



//---------------------------------------------------
void SettLoad(void)
{
	EERestore(&sett.source, 4, 10);
	if ((sett.source < 0) || (sett.source > 1)) 
	//restore default
	{
		sett.source = 0;
		sett.temp = 0;
		sett.range = 0;
		sett.hyst = 1;
		sett.func = 0;
	}
	SettTempGet();
	SettRangeGet();
	SettHystGet();
	SettFuncGet();
	
	/*if ((sett.temp > 120) || (sett.temp < -20)) sett.temp = 0;
	if ((sett.range > 80) || (sett.range < -20)) sett.range = 0;
	if ((sett.hyst > 19) || (sett.hyst < 0)) sett.hyst = 0;
	if ((sett.mode > 1) || (sett.mode < 0)) sett.mode = 0;*/	
}

//---------------------------------------------------
void SettStore(void)
{
	cli();
	EEStore(&sett.source, 4, 10);
	sei();
}

//---------------------------------------------------
int16_t SettSourceGet(void)
{
	if ((sett.source < 0) || (sett.source > 1)) sett.source = 0;	
	return sett.source;
}

//---------------------------------------------------
int16_t SettTempGet(void)
{
	if ((sett.temp > 120) || (sett.temp < -20)) sett.temp = 0;
	return sett.temp;
}

//---------------------------------------------------
int16_t SettRangeGet(void)
{
	if ((sett.range > 80) || (sett.range < -20)) sett.range = 0;
	return sett.range;
}

//---------------------------------------------------
int16_t SettHystGet(void)
{
	if ((sett.hyst > 20) || (sett.hyst < 1)) sett.hyst = 1;
	return sett.hyst;
}

//---------------------------------------------------
int16_t SettFuncGet(void)
{
	if ((sett.func > 1) || (sett.func < 0)) sett.func = 0;
	return sett.func;
}

//---------------------------------------------------
char *CommSource(void)
{
	char *dest = &sett.buff[0];
	*dest = 'U'; dest++;
	*dest = 's'; dest++;
	*dest = 't'; dest++;
	*dest = 'w'; dest++;
	*dest = ':'; dest++;
	
	if (SettSourceGet() == 0)
	{
		*dest = 'P'; dest++;
		*dest = 'o'; dest++;
		*dest = 't'; dest++;
		*dest = 'e'; dest++;
		*dest = 'n'; dest++;
		*dest = '.';
	}
	else
	{
		*dest = 'P'; dest++;
		*dest = 'a'; dest++;
		*dest = 'm'; dest++;
		*dest = 'i'; dest++;
		*dest = 'e'; dest++;
		*dest = 'c'; dest++;
	}
	dest++; *dest = 0;
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommTemp(void)
{
	char *dest = &sett.buff[0];
	*dest = 'T'; dest++;
	*dest = 'e'; dest++;
	*dest = 'm'; dest++;
	*dest = 'p'; dest++;
	*dest = ':'; dest++;
	
	if (SettSourceGet() == 1)
	{
		CommTempValue(SettTempGet(), dest);
	}
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommTempSimple(int16_t temp)
{
	char *dest = &sett.buff[0];
	*dest = ('T'); dest++;
	*dest = ('='); dest++;
	
	CommTempValue(temp, dest);

	dest += 3;
	*dest = ('*'); dest++;
	*dest = ('C'); dest++;
	*dest = 0;
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommVolumeSimple(int16_t temp)
{
	char *dest = &sett.buff[0];

	*dest = ('U'); dest++;
	*dest = ('='); dest++;

	CommTempValue(temp, dest);

	dest += 3;
	*dest = ('*'); dest++;
	*dest = ('C'); dest++;
	*dest = 0;
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommRange(void)
{
	char *dest = &sett.buff[0];
	*dest = 'Z'; dest++;	
	*dest = 'a'; dest++;
	*dest = 'k'; dest++;
	*dest = 'r'; dest++;
	*dest = ':'; dest++;
	
	CommTempValue(SettRangeGet(), dest);	
	dest += 3;
	*dest = '.'; dest++;
	*dest = '.'; dest++;
	*dest = '.'; dest++;

	CommTempValue(SettRangeGet() + NTC_RANGE_LEN, dest);
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommHyst(void)
{
	char *dest = &sett.buff[0];
	*dest = 'H'; dest++;
	*dest = 'i'; dest++;
	*dest = 's'; dest++;
	*dest = 't'; dest++;
	*dest = ':'; dest++;
	
	CommTempValue(SettHystGet(), dest);
	return &sett.buff[0];
}

//---------------------------------------------------
char *CommFunc(void)
{
	char *dest = &sett.buff[0];
	*dest = 'F'; dest++;
	*dest = 'u'; dest++;
	*dest = 'n'; dest++;
	*dest = 'k'; dest++;
	*dest = ':'; dest++;
	
	if (sett.func == 0)
	{
		*dest = 'O'; dest++;
		*dest = 'g'; dest++;
		*dest = 'r'; dest++;
		*dest = 'z'; dest++;
		*dest = 'e'; dest++;
		*dest = 'w'; dest++;
		*dest = '.'; 
	}
	else
	{
		*dest = 'C'; dest++;
		*dest = 'h'; dest++;
		*dest = 'l'; dest++;
		*dest = 'o'; dest++;
		*dest = 'd'; dest++;
		*dest = 'z'; dest++;
		*dest = '.';
	}
	dest++; *dest = 0;
	return &sett.buff[0];
}

//---------------------------------------------------
void CommParamDisp(void)
{

	SuartBufferAdd("\r\n");
	SuartBufferAdd(CommSource());
	SuartBufferSend();

	if (sett.source == 0)		//potentiometer mode
	{
		SuartBufferAdd("\r\n");
		SuartBufferAdd(CommRange());
		SuartBufferSend();

	}
	else						//memory mode
	{
		SuartBufferAdd("\r\n");
		SuartBufferAdd(CommTemp());
		SuartBufferSend();
	}
	SuartBufferAdd("\r\n");
	SuartBufferAdd(CommHyst());
	SuartBufferSend();
	
	SuartBufferAdd("\r\n");
	SuartBufferAdd(CommFunc());
	SuartBufferAdd("\r\n");
	SuartBufferSend();	
}


//---------------------------------------------------
void CommParamSet(char param, int16_t val)
{
	if (param == COM_PARAM_MODE) sett.source = val;
	if (param == COM_PARAM_RANGE) sett.range = val;
	if (param == COM_PARAM_TEMP) sett.temp = val;
	if (param == COM_PARAM_HYST) sett.hyst = val;
	if (param == COM_PARAM_FUNC) sett.func = val;
	
	SettTempGet();
	SettRangeGet();
	SettHystGet();
	SettFuncGet();
	
	SettStore();
	CommParamDisp();
}

//---------------------------------------------------
void CommTempValue(int16_t temp, char *dest)
{
	char d;
	char empty = 0;
		
		if (temp < 0)
		{
			*dest = '-';
			temp *= -1;
		}
		else if (temp >= 100)
		{
			*dest = '1';
			temp -= 100;
			empty = 1;
		}
		else
		*dest = ' ';
		dest++;
		
		d = (temp / 10);
		if ((d > 0) || (empty)) *dest = (d + '0');
		else *dest = ' ';
		dest++;
		
		temp %= 10;
		d = (temp / 1);
		*dest = (d + '0');
		dest++;
		*dest = 0;
}

