/*
 * ee_mem.c
 *
 * Created: 2015-04-24 12:41:08
 *  Author: dsosnowski
 */ 

//---------------------------------------
//includes
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
//#include <avr/pgmspace.h>

#include "ee_mem.h"


//---------------------------------
void EEStore(char *pt, char adr, char len)
{
	while (len)
	{
		EEwrite(adr, *pt);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
void EERestore(char *pt, char adr, char len)
{
	while (len)
	{
		*pt = EEread(adr);
		adr++;
		pt++;
		len--;
	}
}

//---------------------------------
void EEwrite16(uint16_t address, int16_t data16)
{
	EEwrite(address, (char)(data16 & 0xFF));
	data16 >>= 8;
	EEwrite(address + 1, (char)(data16 & 0xFF));
}

//---------------------------------
int16_t EEread16(uint16_t address)
{
	int16_t data16 = 0;
	data16 += EEread(address + 1);
	data16 <<= 8;
	data16 += EEread(address);
	return data16;
}

//---------------------------------
void EEwrite(uint16_t address, char data)
{
	while(EECR & (1<<EEPE)); 		//Wait for completion of previous write
	EECR = 0;						//Set Programming mode

	EEAR = address;					//Set up address and data registers
	EEDR = data;

	EECR |= (1<<EEMPE);				//Write logical one to EEMPE
	EECR |= (1<<EEPE);				//Start eeprom write by setting EEPE
}

//---------------------------------
char EEread(uint16_t address)
{
	while(EECR & (1<<EEPE));		//Wait for completion of previous write
	
	EEAR = address;					//Set up address register
	EECR |= (1<<EERE);				//Start eeprom read by writing EERE
	
	return EEDR;					//Return data from data register
}
