/*
 * eko_term.c
 *
 * Created: 2015-01-20 14:52:47
 *  Author: dsosnowski
 */ 


//---------------------------------------
//includes
#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/sleep.h>
//#include <avr/pgmspace.h>
#include "defs.h"
#include "s_uart.h"
#include "comm.h"
#include "dstemp.h"
#include "wdog.h"
#include "ntctemp.h"

#define MAIN_DELAY			50
#define MAIN_LOOPPERSEC		40

void HardInit(void);



//-----------------------------------------
struct {
	char loop;
	char tick;
	char step;
	char refresh;
	char pause;
} system;

struct {
	char error;
	int16_t delta;
} reg;


//---------------------------------------------
int main(void)
{
	WdtSet(7);
	HardInit();
	SettLoad();	
	SuartInit();
	
	_delay_ms(200);
	SuartBufferAdd("\r\n**TERMOSTAT**\0");
	SuartBufferSend();
	
	CommParamDisp();
	
	NtcInit();
	
    while(1)
    {	
		AnalogProc();

		//------------------------------------------------------------
		if (system.tick)
		{
			system.refresh = 1;
			system.tick = 0;			
		}
				
		//------------------------------------------------------------
		if (system.refresh)
		{
			
			NtcTempCalc(AnalogGet(ANALOG_NTC_CHANNEL));
			NtcVolumeCalc(AnalogGet(ANALOG_VOLUME_CHANNEL));
			AnalogRestart();
			
			//-----------------------------------------------
			//potentiometer
			if (SettSourceGet() == 0)
			{
				if (SettFuncGet() == 0)
					reg.delta = (NtcTempGet() - NtcVolumeGet());
				else
					reg.delta = (NtcVolumeGet() - NtcTempGet());
					
				if (NtcStatusGet() == 0)
				{
					if (reg.delta >= SettHystGet()) OUT_OFF;
					if (reg.delta <= 0) OUT_SET;
				}
				else OUT_OFF;				
			}
			//memory
			else
			{
				if (SettFuncGet() == 0)
					reg.delta = (NtcTempGet() - SettTempGet());
				else
					reg.delta = (SettTempGet() - NtcTempGet());

				if ((NtcStatusGet() & NTC_SENSOR_ERROR) == 0)
				{
					if (reg.delta >= SettHystGet()) OUT_OFF;
					if (reg.delta <= 0) OUT_SET;
				}
				else OUT_OFF;
			}
						
			//-----------------------------------------------
			if (CommandPauseGet() == 0)
			{
				SuartBufferAdd("\r\n");
				SuartBufferAdd(CommTempSimple(NtcTempGet()));		
		
				SuartBufferAdd(",");

				if (SettSourceGet() == 0)
				{
					SuartBufferAdd(CommVolumeSimple(NtcVolumeGet()));			
					if (NtcStatusGet())	SuartBufferAdd(" ERR!");
				}
				else
				{
					SuartBufferAdd(CommVolumeSimple(SettTempGet()));
					if (NtcStatusGet() & NTC_SENSOR_ERROR)	SuartBufferAdd(" ERR!");
				}
				
				if (OUT_GET) SuartBufferAdd(", ON \0");
				else SuartBufferAdd(", OFF\0");
				
		
				SuartBufferSend();			
			}
			system.refresh = 0;		
		}

		//------------------------------------------------------------
		//if no refresh
		else
		{
			if (CommandProc())
			{
				CommParamSet(CommandCodeGet(), CommandValueGet());
				CommandClear();
			}
		}
		
		//------------------------------------------------------------
		_delay_ms(MAIN_DELAY);
		
		if (system.loop < MAIN_LOOPPERSEC)
			system.loop++;
		else
		{
			system.loop = 0;
			system.tick = 1;
		}
						
		WdtReset();
    }
}

//---------------------------------------------
void HardInit(void)
{
	IO_PORT = (1<<TXD_PIN | 1<<RXD_PIN);
	IO_DDR |= (1<<OUT_PIN);
	
	PRR = POWER_CONFIG;
}