


#define F_CPU 12000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "s_uart.h"
#include "defs.h"

#define CORE_CLOCK		8000000
#define BAUD_RATE		9600

#define TIMER_DIV		8
#define TIMER_CLOCK		(CORE_CLOCK/TIMER_DIV)
#define BAUD_FACTOR		(TIMER_CLOCK/BAUD_RATE)	//1 int on 1 bit

#define SUART_PORT		IO_PORT
#define SUART_PIN		IO_PIN
#define SUART_DDR		IO_DDR
#define TXD 			TXD_PIN
#define RXD 			RXD_PIN
//#define TEN			TEN_PIN

#define TXD_SET			SUART_PORT |= (1<<TXD)
#define TXD_CLR			SUART_PORT &= ~(1<<TXD)
//#define TXD_EN		SUART_PORT |= (1<<TEN)
//#define TXD_DIS		SUART_PORT &= ~(1<<TEN)

#define START_TIMER		TCCR1 = (1<<CS12 | 1<<PWM1A | 1<<CTC1)
#define STOP_TIMER		TCCR1 = (0<<CS12 | 1<<PWM1A | 1<<CTC1)
#define TIMER_VAL		TCNT1
#define CLEAR_TIMER		TCNT1 = 0

#define DISABLE_RXD		GIMSK = 0
#define CLEAR_RXD	 	(GIFR = (1<<INTF0))
#define ENABLE_RXD		(GIMSK = (1<<INT0))

//i.e        |         |          |
//"VV" = -----__--_-_-_-__--_-_-_------


#define SUART_STATUS_IDLE	0		//IDLE
#define SUART_STATUS_TX		1		//TX
#define SUART_STATUS_RX		2		//RX




//-------------------------------------------
struct {
	volatile char status;
	
	volatile char s_byte;
	volatile char s_index;
	char s_counter;	
	char s_buffer[SUART_SEND_BUFFER];
	
	volatile char r_index;
	volatile char r_counter;
	volatile char r_buffer[SUART_READ_BUFFER];
} suart;

struct {		
	volatile char status;
	char code;
	int16_t val;
	char ready;
	volatile char pause;
} command;

//-------------------------------------------
void SuartInit(void)
{
	 SUART_PORT |= (1<<TXD | 1<<RXD);	//config ports
     SUART_DDR |= (1<<TXD);
	 //SUART_PORT |= (1<<TXD | 1<<RXD | 1<<<TEN);	//config ports
     //SUART_DDR |= (1<<TXD | 1<<TEN);
   
	 MCUCR |= (1<<ISC01);
   
	 STOP_TIMER;
	 OCR1C = BAUD_FACTOR;				//S_uart freq  

	 TIFR |= (1<<TOV1);					//clear flag
 	 TIMSK |= (1<<TOIE1);				//enable int		

	 CLEAR_RXD;							//clear flag
	 ENABLE_RXD;						//INT0
	
	 sei();
}

//-------------------------------------------
void SuartSendData(char *data)
{
	DISABLE_RXD;
	//if (*data) TXD_EN;	

	//_delay_ms(2);

	CLEAR_TIMER;					//first - idle cycle
	START_TIMER;					//start

	while(*data)
	{
 		suart.status = SUART_STATUS_TX;			//transmit mode
		suart.s_byte = 0;			//start bit first
		suart.s_index = 10;			//set cycles
		while(suart.s_index >= 10);	//wait for star bit
		suart.s_byte = *data;		//data
		while(suart.s_index > 1);		//wait for complete and next
		suart.s_byte = 0xFF;			//stop_bit
		while(suart.s_index > 0);		//wait for complete and next
		data++;						//next data
	}
	suart.status = SUART_STATUS_IDLE;				//receive mode
	STOP_TIMER;

	//TXD_DIS;
	CLEAR_RXD;
	ENABLE_RXD;	
}

//-------------------------------------------
void SuartRClear(void)
{
	suart.r_buffer[0] = 0;
	suart.r_counter = 0;
	suart.status = SUART_STATUS_IDLE;				//receive mode
}

//-------------------------------------------
void SuartSClear(void)
{
	suart.s_counter = 0;
	suart.s_buffer[0] = 0;
}

//-------------------------------------------
SIGNAL(TIM1_OVF_vect)
{	      
	static char r_byte = 0xFF;
	
	//---------------------------------------------
	if (suart.status == SUART_STATUS_TX)		//if any byte to send
	{
		if (suart.s_byte & 1) TXD_SET; 
		else		   		  TXD_CLR;
		suart.s_byte>>=1;			//next bit
		suart.s_index--;			//decrement counter
	}
	
	//---------------------------------------------
	if (suart.status == SUART_STATUS_RX)		//if read in progress			
	{
		//---------------------------------------------
		if (suart.r_index < 10)
		{		
			suart.r_index++;								//increment counter
			if (SUART_PIN & (1<<RXD)) 
			{
				if (SUART_PIN & (1<<RXD))r_byte += 0x80;		//set bit 7	
			}
			if (suart.r_index == 9)
			{
				command.pause = 3;
				if (r_byte == COMMAND_CHAR) suart.r_counter = 0;
				if (r_byte == ACCEPT_CHAR)  command.status = 1;
				suart.r_buffer[suart.r_counter] = r_byte;
				if (suart.r_counter < (SUART_READ_BUFFER - 1)) suart.r_counter++;
				suart.r_buffer[suart.r_counter] = 0;		
			}
			r_byte >>= 1;									//at the end - shift
		} 												//control counter - max bits
		//---------------------------------------------
		if (suart.r_index >= 10)
		{
			suart.status = SUART_STATUS_IDLE;
			STOP_TIMER;
			CLEAR_RXD;
			ENABLE_RXD;	
		}
	}
}

//-------------------------------------------
SIGNAL(INT0_vect)
{	 
	//if ((SUART_PIN & (1<<RXD)) == 0)     
	//{
		DISABLE_RXD;
		suart.status = SUART_STATUS_RX;
		suart.r_index = 0;
		TIMER_VAL = (BAUD_FACTOR / 2);
		START_TIMER;
	//}	
}

//-------------------------------------------
void SuartBufferSend(void)
{
	suart.s_buffer[(SUART_SEND_BUFFER - 1)] = 0;
	SuartSendData(suart.s_buffer);
	SuartSClear();
}

//-------------------------------------------
void SuartBufferAdd(char *src)
{
	char enough = 0;
	
	while (enough == 0)
	{
		if (suart.s_counter < SUART_SEND_BUFFER)
		{
			if ((*src == 0) || (src == 0))
				enough = 1;
			else
			{
				suart.s_buffer[suart.s_counter] = *src;
				src++;
				suart.s_counter++;
				if (suart.s_counter < SUART_SEND_BUFFER)
					suart.s_buffer[suart.s_counter] = 0;
				else
					suart.s_buffer[(SUART_SEND_BUFFER - 1)] = 0;
			}
		}
		else
		{
			enough = 1;
		}
	}
}

//-------------------------------------------
char *SuartBufferSet(int16_t index)
{
	suart.s_counter = index;
	return &suart.s_buffer[suart.s_counter];
}


//-------------------------------------------
char CommandProc(void)
{
	if (command.status)
	{		
		command.ready = COMMAND_LEVEL_IDLE;
		if (suart.r_buffer[COMMAND_START_INDEX] == COMMAND_CHAR)
		{
			if (suart.r_buffer[COMMAND_CODE_INDEX] == COMMAND_RETURN)
			{
				command.code = COMMAND_RETURN;
				command.val = 0; 
				command.ready = COMMAND_LEVEL_RET;
			}
			else
			{
				if (suart.r_buffer[COMMAND_EQUAL_INDEX] == COMMAND_VAL)
				{
					if (CommandValue(COMMAND_VALUE_INDEX))
					{
						command.code = suart.r_buffer[COMMAND_CODE_INDEX];
						command.ready = COMMAND_LEVEL_VAL;						
					}
				}
			}
		}

		SuartRClear();
		command.status = 0;
	}
	return command.ready;
}
//------------------------------------------
char CommandValue(char index)
{
	char result = 0;
	char minus = 0;
	int16_t d1 = 0;
	char d2 = 0;
	char d3 = 0;

	//minus
	if (suart.r_buffer[index] == '-')
	{
		minus = 1;
		index++;
	}

	if ((suart.r_buffer[index] >= '0') && (suart.r_buffer[index] <= '9'))
	{
		d1 = (suart.r_buffer[index] - '0');
		index++;
		
		if ((suart.r_buffer[index] >= '0') && (suart.r_buffer[index] <= '9'))
		{
			d1 *= 10;
			d2 = (suart.r_buffer[index] - '0');
			index++;
		
			if ((suart.r_buffer[index] >= '0') && (suart.r_buffer[index] <= '9'))
			{
				d1 *= 10;
				d2 *= 10;
				d3 = (suart.r_buffer[index] - '0');
				index++;
			}		
		}
		d1 += d2;
		d1 += d3;
		if (minus) d1 *= -1;
		
		command.val = d1;
		result = 1;
	}
	else
		result = 0;

	return result;
}

//------------------------------------------
void CommandClear(void)
{
	command.code = 0;
	command.val = 0;
	command.ready = 0;
}

//------------------------------------------
char CommandCodeGet(void)
{
	return command.code;
}

//------------------------------------------
int16_t CommandValueGet(void)
{
	return command.val;
}

//------------------------------------------
char CommandPauseGet(void)
{
	if (command.pause) command.pause--;
	return command.pause;
}

/*
//------------------------------------------
void Value2str(char *dest, char val)
{
	char digit;
	char temp;
	digit = (val/100);
	*dest = (digit + '0'); dest++;
	temp = (val%100);
	digit = (temp/10);
	*dest = (digit + '0'); dest++;
	digit = (temp%10);
	*dest = (digit + '0'); 
}

//------------------------------------------
void SmallValue2str(char *dest, char val)
{
	char digit;
	if (val > 99) val = 99;
	digit = (val/10);
	*dest = (digit + '0'); dest++;
	digit = (val%10);
	*dest = (digit + '0');
}*/

